/*********************************************************************
 *
 *                Microchip USB C18 Firmware - Mouse Demo
 *
 *********************************************************************
 * FileName:        user_mouse.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the CompanyE for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS ISECONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Rawin Rojvanit       11/19/04    Original.
 ********************************************************************/

/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include <usart.h>
#include "system\typedefs.h"

#include "system\usb\usb.h"

#include "io_cfg.h"             // I/O pin mapping
#include "user\user_mouse.h"
#include "delays.h"

/** V A R I A B L E S ********************************************************/
#pragma udata
/*
byte old_sw2,old_sw3;

BOOL emulate_mode;
rom signed char dir_table[]={-4,-4,-4, 0, 4, 4, 4, 0};
byte movement_length;
byte vector = 0;
*/

char buffer[65];
/** P R I V A T E  P R O T O T Y P E S ***************************************/
/*
void BlinkUSBStatus(void);
BOOL Switch2IsPressed(void);
BOOL Switch3IsPressed(void);

void Emulate_Mouse(void);
*/
void InitLCD(void);
void WriteLCD(char);
void BusyLCD(void);
void ClearLCD(void);
void SetAdrsCLD(char);
void CmdCLD(char);

/** D E C L A R A T I O N S **************************************************/
#pragma code
void UserInit(void)
{
/*
    mInitAllLEDs();
    mInitAllSwitches();
    old_sw2 = sw2;
    old_sw3 = sw3;

    buffer[0]=0;
    buffer[1]=0;
    buffer[2]=0;
    buffer[3]=0;
    
    emulate_mode = TRUE;
*/
	InitLCD();
}//end UserInit

/******************************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user routines.
 *                  It is a mixture of both USB and non-USB tasks.
 *
 * Note:            None
 *****************************************************************************/
void ProcessIO(void)
{
	char *pointer;
    if((usb_device_state < CONFIGURED_STATE)||(UCONbits.SUSPND==1)) return;
 
   if (HIDRxReport(buffer, EP1_BUF_SIZE) > 0) // USB receive buffer has data
    {
        while(mHIDTxIsBusy()){}                 // blocking
        HIDTxReport(buffer, EP1_BUF_SIZE);   // transmit packet
		ClearLCD();
		pointer=buffer;
		SetAdrsCLD(0x00);
		while(*pointer) WriteLCD(*(pointer++));// regular order
		SetAdrsCLD(0x40);
		do WriteLCD(*(--pointer)); while(pointer!=buffer);// reverse order
	}

}//end ProcessIO


/*
	Connections;

	DB0:BD7 -- RB0:RB7	(Input/Output)
	E       -- RC0		(Output)
	RS      -- RC1		(Output)
	R/W     -- RC2		(Output)
*/

void InitLCD(void)
{
	TRISB = 0x00;
	TRISC  = 0x00;
	PORTCbits.RC0 = 0;	//E low

	Delay1KTCYx(80);	//16 ms (Delay of more than 15ms)
	CmdCLD(0x30);		//Function reset 8bits
	Delay1KTCYx(30);	//6 ms (Delay of more than 4.1ms)
	CmdCLD(0x30);		//Function reset 8bits
	CmdCLD(0x30);		//Function reset 8bits
	CmdCLD(0x38);		//Fuction set
	CmdCLD(0x08);		//Display off D=C=B=0
	CmdCLD(0x0C);		//Display on D=1 C=B=0
	CmdCLD(0x06);		//Entry I/D=1 S=0
}

void WriteLCD(char datum)
{
	PORTB=datum;
	PORTCbits.RC2 = 0;	//R/W low
	PORTCbits.RC1 = 1;	//RS high
	PORTCbits.RC0 = 1;	//E high
	PORTCbits.RC0 = 0;	//E low
	BusyLCD();
}

void BusyLCD(void)
{
	do{

		TRISB=0xFE;
		PORTCbits.RC2 = 1;	//R/W high
		PORTCbits.RC1 = 0;	//RS low
		PORTCbits.RC0 = 1;	//E high
		PORTCbits.RC0 = 0;	//E low
	}while (PORTBbits.RB7==1);
	PORTCbits.RC2 = 0;	//R/W low
	TRISB=0x00;
}

void ClearLCD(void)
{
	CmdCLD(0x01);
}

void SetAdrsCLD(char address)
{
	address|=0x80;
	CmdCLD(address);
}

void CmdCLD(char cmd)
{
	PORTB=cmd;
	PORTCbits.RC2 = 0;	//R/W low
	PORTCbits.RC1 = 0;	//RS low
	PORTCbits.RC0 = 1;	//E high
	PORTCbits.RC0 = 0;	//E low
	BusyLCD();
}
/** EOF user_mouse.c *********************************************************/
